-- Time-stamp: <2006-12-08 16:05:10 cklin>

module Infopipes.Polarity where

import Data.List
import Infopipes.Syntax
import Infopipes.Utils

type Merger = [[Port]] -> [Port]

--

alt_lift_stmt :: Statement -> [Statement]
alt_lift_stmt = lift_stmt where
    lift_stmt (Alt choices)    = concatMap lift_stmt choices
    lift_stmt (Block u v body) = mapx2 (Block u v) lift_stmt body
    lift_stmt (Cond branches)  = mapx2 Cond lift_branch branches
    lift_stmt simple_stmt      = [simple_stmt]
    lift_branch (u, body) = map ((,) u) (lift_stmt body)
    mapx2 f g = map f . cartesian . map g

stmt_ports :: Merger -> Merger -> Statement -> [Port]
stmt_ports block_merge cond_merge = ports where
    ports (Single _)        = []
    ports (Block _ _ body)  = block_ports body
    ports (Cond branches)   = cond_ports branches
    ports (Input _ port _)  = [(Inport, port)]
    ports (Output port _ _) = [(Outport, port)]
    block_ports = block_merge . map ports
    cond_ports  = cond_merge . map (ports . snd)

list_used_ports = stmt_ports unions unions
list_full_ports = stmt_ports unions intersects
list_uniq_ports = once . stmt_ports concat multi_unions where
    multi_unions xs = foldl multi_union [] xs
    multi_union us vs = us ++ (vs \\ us)

list_poly_ports :: [Statement] -> [[Port]]
list_poly_ports alt_paths = map intersects candidates where
    single = map (intersect $ once $ concat used) used
    used = map list_used_ports alt_paths
    full = map list_full_ports alt_paths
    uniq = map list_uniq_ports alt_paths
    candidates = transpose [single, full, uniq]

