-- Time-stamp: <2006-12-08 16:05:43 cklin>

module Infopipes.Tests.Polarity where

import Test.HUnit
import Infopipes.Syntax
import Infopipes.Polarity

--

cmd_0 = Single "a"
cmd_1 = Single "b"
cmd_2 = Single "c"
cmd_3 = Single "d"

in_0 = PortName "in0"
in_1 = PortName "in1"
out_0 = PortName "out0"
out_1 = PortName "out1"

inport_0 = (Inport, in_0)
inport_1 = (Inport, in_1)
outport_0 = (Outport, out_0)
outport_1 = (Outport, out_1)

input_0 = Input "w" in_0 ";"
input_1 = Input "x" in_1 ";"
output_0 = Output out_0 "y+z" ";"
output_1 = Output out_1 "y-z" ";"

alt = Alt
block = Block "{" "}"
cond = Cond . map ((,) ",")

--

alt_lift_stmt_tests = TestLabel "alt_lift_stmt" $ TestList
    [alt_lift_stmt_0, alt_lift_stmt_1, alt_lift_stmt_2,
     alt_lift_stmt_3, alt_lift_stmt_4, alt_lift_stmt_5]

alt_lift_stmt_0 = TestCase $ answer @=? alt_lift_stmt input
    where input  = cmd_0
          answer = [input]

alt_lift_stmt_1 = TestCase $ answer @=? alt_lift_stmt input
    where input  = block [cmd_0, cmd_1]
          answer = [input]

alt_lift_stmt_2 = TestCase $ answer @=? alt_lift_stmt input
    where input  = block [cmd_0, alt [cmd_1, cmd_2]]
          answer = [block [cmd_0, cmd_1], block [cmd_0, cmd_2]]

alt_lift_stmt_3 = TestCase $ answer @=? alt_lift_stmt input
    where input  = block [alt [cmd_1, cmd_2], cmd_3]
          answer = [block [cmd_1, cmd_3], block [cmd_2, cmd_3]]

alt_lift_stmt_4 = TestCase $ answer @=? alt_lift_stmt input
    where input  = alt [cmd_1, alt [cmd_2, cmd_3]]
          answer = [cmd_1, cmd_2, cmd_3]

alt_lift_stmt_5 = TestCase $ answer @=? alt_lift_stmt input
    where input  = cond [cmd_1, alt [cmd_2, cmd_3]]
          answer = [cond [cmd_1, cmd_2], cond [cmd_1, cmd_3]]

--

list_used_ports_tests = TestLabel "list_used_ports" $ TestList
    [list_used_ports_0, list_used_ports_1]

list_used_ports_0 = TestCase $ answer @=? list_used_ports input
    where input  = block [input_0, input_1, output_0]
          answer = [inport_0, inport_1, outport_0]

list_used_ports_1 = TestCase $ answer @=? list_used_ports input
    where input  = cond [input_0, output_0, input_1]
          answer = [inport_0, outport_0, inport_1]

--

list_full_ports_tests = TestLabel "list_full_ports" $ TestList
    [list_full_ports_0, list_full_ports_1, list_full_ports_2,
     list_full_ports_3, list_full_ports_4, list_full_ports_5,
     list_full_ports_5]

list_full_ports_0 = TestCase $ answer @=? list_full_ports input
    where input  = block [input_0, input_1, output_0, input_1]
          answer = [inport_0, inport_1, outport_0]

list_full_ports_1 = TestCase $ answer @=? list_full_ports input
    where input  = cond [input_0, output_0, input_1]
          answer = []

list_full_ports_2 = TestCase $ answer @=? list_full_ports input
    where input  = cond [input_0]
          answer = [inport_0]

list_full_ports_3 = TestCase $ answer @=? list_full_ports input
    where input  = cond [input_0, block [input_0, output_0]]
          answer = [inport_0]

list_full_ports_4 = TestCase $ answer @=? list_full_ports input
    where input  = cond [input_0, cond [input_0, output_0]]
          answer = []

list_full_ports_5 = TestCase $ answer @=? list_full_ports input
    where input  = cond [input_0, cond [output_0, output_0]]
          answer = []

list_full_ports_6 = TestCase $ answer @=? list_full_ports input
    where input  = cond [input_0, cond [input_0, input_0]]
          answer = [inport_0]

--

list_uniq_ports_tests = TestLabel "list_uniq_ports" $ TestList
    [list_uniq_ports_0, list_uniq_ports_1, list_uniq_ports_2,
     list_uniq_ports_3]

list_uniq_ports_0 = TestCase $ answer @=? list_uniq_ports input
    where input  = block [input_0, input_1, output_0]
          answer = [inport_0, inport_1, outport_0]

list_uniq_ports_1 = TestCase $ answer @=? list_uniq_ports input
    where input  = block [input_0, input_1, output_0, input_1]
          answer = [inport_0, outport_0]

list_uniq_ports_2 = TestCase $ answer @=? list_uniq_ports input
    where input  = cond [input_0, block [input_0, input_1]]
          answer = [inport_0, inport_1]

list_uniq_ports_3 = TestCase $ answer @=? list_uniq_ports input
    where input  = cond [input_1, block [input_0, input_0]]
          answer = [inport_1]

--

list_poly_ports_tests = TestLabel "list_poly_ports" $ TestList
    [list_poly_ports_0, list_poly_ports_1, list_poly_ports_2,
     list_poly_ports_3, list_poly_ports_4]

list_poly_ports_0 = TestCase $ answer @=? list_poly_ports input
    where input  = [block [input_0, input_1, output_0]]
          answer = [[inport_0, inport_1, outport_0]]

list_poly_ports_1 = TestCase $ answer @=? list_poly_ports input
    where input  = [block [input_0, input_1, output_0, input_1]]
          answer = [[inport_0, outport_0]]

list_poly_ports_2 = TestCase $ answer @=? list_poly_ports input
    where input  = [cond [block [input_0, output_1], input_0]]
          answer = [[inport_0]]

list_poly_ports_3 = TestCase $ answer @=? list_poly_ports input
    where input  = [input_0, output_1]
          answer = [[inport_0], [outport_1]]

list_poly_ports_4 = TestCase $ answer @=? list_poly_ports input
    where input  = [branch, output_1]
          branch = block [input_0, cond [output_0, output_1]]
          answer = [[inport_0], []]
--

unit_tests = runTestTT $ TestList
    [alt_lift_stmt_tests,
     list_used_ports_tests, list_full_ports_tests,
     list_uniq_ports_tests, list_poly_ports_tests]

